@@ -61,6 +61,7 @@ typedef enum
     SRC_PRESENTATION_STATE_EXPANDED,
     SRC_PRESENTATION_STATE_COLLAPSED,
     SRC_PRESENTATION_STATE_MINIMIZED,
+    SRC_PRESENTATION_STATE_SELECTED,
     SRC_PRESENTATION_CHILDREN_COUNT,
     SRC_PRESENTATION_VALUE_CRT,
     SRC_PRESENTATION_COLUMN_HEADER,
@@ -220,6 +221,7 @@ src_presentation_get_item_type (const gchar *item)
 	    {SRC_PRESENTATION_TEXT_SELECTIONS,	"text:selections"},
 	    {SRC_PRESENTATION_TEXT_DIFFERENCE,	"text:difference"},
 	    {SRC_PRESENTATION_STATE_CHECKED,	"state:checked"},
+	    {SRC_PRESENTATION_STATE_SELECTED,	"state:selected"},
 	    {SRC_PRESENTATION_STATE_UNCHECKED,	"state:unchecked"},
 	    {SRC_PRESENTATION_STATE_ENABLED,	"state:enabled"},
 	    {SRC_PRESENTATION_STATE_EXPANDED,	"state:expanded"},
@@ -1042,6 +1044,13 @@ src_presentation_get_info_from_sro (SRObject *obj,
 		    rv = state & SR_STATE_CHECKED ? TRUE : FALSE;
 	    }
 	    break;
+	case SRC_PRESENTATION_STATE_SELECTED:
+	    {
+		SRState state;
+		if (sro_get_state (obj, &state, SR_INDEX_CONTAINER))
+		    rv = state & SR_STATE_SELECTED ? TRUE : FALSE;
+	    }
+	    break;
 	case SRC_PRESENTATION_STATE_UNCHECKED:
 	    {
 		SRState state;
@@ -1110,8 +1119,12 @@ src_presentation_get_info_from_sro (SRObject *obj,
 		
 		if (last_offset == crt_offset)
 		    last_offset = last_offset2;
-		if (abs (crt_offset - last_offset) == 1)
+		if (abs (crt_offset - last_offset) == 1 &&
+		    sro_text_is_same_line (obj, last_offset, SR_INDEX_CONTAINER))
 		    type = SR_TEXT_BOUNDARY_CHAR;
+		else if (abs (crt_offset - last_offset) == 1 &&
+		         !sro_text_is_same_line (obj, last_offset, SR_INDEX_CONTAINER))
+		    type = SR_TEXT_BOUNDARY_LINE;    
 		else if (sro_text_is_same_line (obj, last_offset, SR_INDEX_CONTAINER) ||
 		         sro_is_word_navigation (obj, crt_offset, last_offset, SR_INDEX_CONTAINER))
 		    type = SR_TEXT_BOUNDARY_WORD;
@@ -1415,6 +1428,10 @@ src_presentation_sro_get_pres_for_magnifier (SRObject *obj,
     gchar *role;
     SRCPresentationBoundsType btype;
     gchar *presentation_ = NULL;
+    gint default_int_val = 0;
+    
+    static gint size_x = 0,
+	        size_y = 0;
 
     sru_assert (obj && reason);
 
@@ -1428,8 +1445,29 @@ src_presentation_sro_get_pres_for_magnifier (SRObject *obj,
 	switch (btype)
 	{
 	    case  SRC_PRESENTATION_BOUNDS_TYPE_OBJECT:
-		    sro_get_location (obj, SR_COORD_TYPE_SCREEN, &location, SR_INDEX_CONTAINER);
-		    break;
+		if (size_x == 0)
+		{
+    		    srconf_get_data_with_default (MAGNIFIER_DISPLAY_SIZE_X, 
+		    		    	          CFGT_INT, 
+				    	    	  (gpointer)&size_x, 
+				    	    	  (gpointer)&default_int_val, 
+				    	    	  MAGNIFIER_CONFIG_PATH);
+		}
+
+		if (size_y == 0)
+		{
+		    srconf_get_data_with_default (MAGNIFIER_DISPLAY_SIZE_Y, 
+				    	          CFGT_INT, 
+				    	    	  (gpointer)&size_y, 
+				    	    	  (gpointer)&default_int_val, 
+				    	    	  MAGNIFIER_CONFIG_PATH);
+		}
+
+		sro_get_location (obj, SR_COORD_TYPE_SCREEN, &location, SR_INDEX_CONTAINER);
+		if ((location.width - location.x)  >= size_x + 1 &&
+    		    (location.height - location.y) >= size_y + 1)
+			get = FALSE;
+		break;
 	    case SRC_PRESENTATION_BOUNDS_TYPE_TEXT_CHAR:
 		    sro_text_get_caret_location (obj, SR_COORD_TYPE_SCREEN, 
 				&location, SR_INDEX_CONTAINER);
@@ -1444,6 +1482,7 @@ src_presentation_sro_get_pres_for_magnifier (SRObject *obj,
 		sru_assert_not_reached ();
 		break;	    
 	}
+		
 	if (get)
 	{
 	    presentation_ = g_strdup_printf ("ROILeft =\"%d\" ROITop =\"%d\" ROIWidth =\"%d\" ROIHeight=\"%d\"",
