@@ -1653,9 +1653,6 @@ src_speech_terminate ()
     g_ptr_array_add (src_created_voices, NULL);
     g_strfreev ((gchar**)g_ptr_array_free (src_created_voices, TRUE));
     
-    gboolean use_speech = TRUE;
-    if (speech_timeout_id == 0)
-        SET_SRCORE_CONFIG_DATA (SRCORE_SPEECH_ACTIVE, CFGT_BOOL, &use_speech);
 }
 
 #define src_speech_is_separator(x) (( x == '.' || x == ';' || x == '?' || x == ',' || x == '!' ) ? TRUE : FALSE)
